package java.awt;

import java.io.ObjectOutputStream;
import java.io.IOException;
import java.util.Enumeration;
import java.util.Hashtable;
import java.io.ObjectInputStream;
import java.io.ObjectStreamField;
import java.util.Vector;
import java.io.Serializable;

public class CardLayout implements LayoutManager2, Serializable
{
    private static final long serialVersionUID = -4328196481005934313L;
    Vector<CardLayout.Card> vector;
    int currentCard;
    int hgap;
    int vgap;
    private static final ObjectStreamField[] serialPersistentFields;
    
    public CardLayout() {
        this(0, 0);
    }
    
    public CardLayout(int hgap, int vgap) {
        this.vector = new Vector();
        this.currentCard = 0;
        this.hgap = hgap;
        this.vgap = vgap;
    }
    
    public int getHgap() {
        return this.hgap;
    }
    
    public void setHgap(int hgap) {
        this.hgap = hgap;
    }
    
    public int getVgap() {
        return this.vgap;
    }
    
    public void setVgap(int vgap) {
        this.vgap = vgap;
    }
    
    @Override
    public void addLayoutComponent(Component component, Object o) {
        synchronized (component.getTreeLock()) {
            if (o == null) {
                o = "";
            }
            if ((o instanceof String)) {
                this.addLayoutComponent((String)o, component);
            }
            else {
                throw new IllegalArgumentException("cannot add to layout: constraint must be a string");
            }
        }
    }
    
    @Deprecated
    @Override
    public void addLayoutComponent(String anObject, Component comp) {
        synchronized (comp.getTreeLock()) {
            if (!this.vector.isEmpty()) {
                comp.setVisible(false);
            }
            int i = 0;
            while (i < this.vector.size()) {
                if (((CardLayout.Card)this.vector.get(i)).name.equals(anObject)) {
                    ((CardLayout.Card)this.vector.get(i)).comp = comp;
                    return;
                }
                ++i;
            }
            this.vector.add(new CardLayout.Card(anObject, comp));
        }
    }
    
    @Override
    public void removeLayoutComponent(Component component) {
        synchronized (component.getTreeLock()) {
            int i = 0;
            while (i < this.vector.size()) {
                if (((CardLayout.Card)this.vector.get(i)).comp == component) {
                    if (component.isVisible() && component.getParent() != null) {
                        this.next(component.getParent());
                    }
                    this.vector.remove(i);
                    if (this.currentCard > i) {
                        --this.currentCard;
                        break;
                    }
                    break;
                }
                else {
                    ++i;
                }
            }
        }
    }
    
    @Override
    public Dimension preferredLayoutSize(Container container) {
        synchronized (container.getTreeLock()) {
            Insets insets = container.getInsets();
            int componentCount = container.getComponentCount();
            int width = 0;
            int height = 0;
            for (int i = 0; i < componentCount; ++i) {
                Dimension preferredSize = container.getComponent(i).getPreferredSize();
                if (preferredSize.width > width) {
                    width = preferredSize.width;
                }
                if (preferredSize.height > height) {
                    height = preferredSize.height;
                }
            }
            return new Dimension(insets.left + insets.right + width + this.hgap * 2, insets.top + insets.bottom + height + this.vgap * 2);
        }
    }
    
    @Override
    public Dimension minimumLayoutSize(Container container) {
        synchronized (container.getTreeLock()) {
            Insets insets = container.getInsets();
            int componentCount = container.getComponentCount();
            int width = 0;
            int height = 0;
            for (int i = 0; i < componentCount; ++i) {
                Dimension minimumSize = container.getComponent(i).getMinimumSize();
                if (minimumSize.width > width) {
                    width = minimumSize.width;
                }
                if (minimumSize.height > height) {
                    height = minimumSize.height;
                }
            }
            return new Dimension(insets.left + insets.right + width + this.hgap * 2, insets.top + insets.bottom + height + this.vgap * 2);
        }
    }
    
    @Override
    public Dimension maximumLayoutSize(Container container) {
        return new Dimension(Integer.MAX_VALUE, Integer.MAX_VALUE);
    }
    
    @Override
    public float getLayoutAlignmentX(Container container) {
        return 0.5f;
    }
    
    @Override
    public float getLayoutAlignmentY(Container container) {
        return 0.5f;
    }
    
    @Override
    public void invalidateLayout(Container container) {
    }
    
    @Override
    public void layoutContainer(Container container) {
        synchronized (container.getTreeLock()) {
            Insets insets = container.getInsets();
            int componentCount = container.getComponentCount();
            boolean n = false;
            for (int i = 0; i < componentCount; ++i) {
                Component component = container.getComponent(i);
                component.setBounds(this.hgap + insets.left, this.vgap + insets.top, container.width - (this.hgap * 2 + insets.left + insets.right), container.height - (this.vgap * 2 + insets.top + insets.bottom));
                if (component.isVisible()) {
                    n = true;
                }
            }
            if (n == 0 && componentCount > 0) {
                container.getComponent(0).setVisible(true);
            }
        }
    }
    
    void checkLayout(Container container) {
        if (container.getLayout() != this) {
            throw new IllegalArgumentException("wrong parent for CardLayout");
        }
        return;
    }
    
    public void first(Container container) {
        synchronized (container.getTreeLock()) {
            this.checkLayout(container);
            int componentCount = container.getComponentCount();
            int i = 0;
            while (i < componentCount) {
                Component component = container.getComponent(i);
                if (component.isVisible()) {
                    component.setVisible(false);
                    break;
                }
                ++i;
            }
            if (componentCount > 0) {
                this.currentCard = 0;
                container.getComponent(0).setVisible(true);
                container.validate();
            }
        }
    }
    
    public void next(Container container) {
        synchronized (container.getTreeLock()) {
            this.checkLayout(container);
            int componentCount = container.getComponentCount();
            int i = 0;
            while (i < componentCount) {
                Component component = container.getComponent(i);
                if (component.isVisible()) {
                    component.setVisible(false);
                    this.currentCard = (i + 1) % componentCount;
                    container.getComponent(this.currentCard).setVisible(true);
                    container.validate();
                    return;
                }
                ++i;
            }
            this.showDefaultComponent(container);
        }
    }
    
    public void previous(Container container) {
        synchronized (container.getTreeLock()) {
            this.checkLayout(container);
            int componentCount = container.getComponentCount();
            int i = 0;
            while (i < componentCount) {
                Component component = container.getComponent(i);
                if (component.isVisible()) {
                    component.setVisible(false);
                    this.currentCard = ((i > 0) ? (i - 1) : (componentCount - 1));
                    container.getComponent(this.currentCard).setVisible(true);
                    container.validate();
                    return;
                }
                ++i;
            }
            this.showDefaultComponent(container);
        }
    }
    
    void showDefaultComponent(Container container) {
        if (container.getComponentCount() > 0) {
            this.currentCard = 0;
            container.getComponent(0).setVisible(true);
            container.validate();
        }
    }
    
    public void last(Container container) {
        synchronized (container.getTreeLock()) {
            this.checkLayout(container);
            int componentCount = container.getComponentCount();
            int i = 0;
            while (i < componentCount) {
                Component component = container.getComponent(i);
                if (component.isVisible()) {
                    component.setVisible(false);
                    break;
                }
                ++i;
            }
            if (componentCount > 0) {
                this.currentCard = componentCount - 1;
                container.getComponent(this.currentCard).setVisible(true);
                container.validate();
            }
        }
    }
    
    public void show(Container container, String anObject) {
        synchronized (container.getTreeLock()) {
            this.checkLayout(container);
            Component comp = null;
            int size = this.vector.size();
            int i = 0;
            while (i < size) {
                CardLayout.Card card = (CardLayout.Card)this.vector.get(i);
                if (card.name.equals(anObject)) {
                    comp = card.comp;
                    this.currentCard = i;
                    break;
                }
                ++i;
            }
            if (comp != null && !comp.isVisible()) {
                int componentCount = container.getComponentCount();
                int j = 0;
                while (j < componentCount) {
                    Component component = container.getComponent(j);
                    if (component.isVisible()) {
                        component.setVisible(false);
                        break;
                    }
                    ++j;
                }
                comp.setVisible(true);
                container.validate();
            }
        }
    }
    
    @Override
    public String toString() {
        return this.getClass().getName() + "[hgap=" + this.hgap + ",vgap=" + this.vgap + "]";
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws ClassNotFoundException, IOException {
        ObjectInputStream.GetField fields = objectInputStream.readFields();
        this.hgap = fields.get("hgap", 0);
        this.vgap = fields.get("vgap", 0);
        if (fields.defaulted("vector")) {
            Hashtable<Object, Object> hashtable = (Hashtable)fields.get("tab", null);
            this.vector = new Vector();
            if (hashtable != null && !hashtable.isEmpty()) {
                Enumeration<Object> keys = hashtable.keys();
                while (keys.hasMoreElements()) {
                    String key = (String)keys.nextElement();
                    Component component = (Component)hashtable.get(key);
                    this.vector.add(new CardLayout.Card(key, component));
                    if (component.isVisible()) {
                        this.currentCard = this.vector.size() - 1;
                    }
                    else {
                        continue;
                    }
                }
            }
        }
        else {
            this.vector = (Vector)fields.get("vector", null);
            this.currentCard = fields.get("currentCard", 0);
        }
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        Hashtable<CardLayout.Card, CardLayout.Card> hashtable = new Hashtable();
        for (int size = this.vector.size(), i = 0; i < size; ++i) {
            CardLayout.Card card = (CardLayout.Card)this.vector.get(i);
            hashtable.put(card.name, card.comp);
        }
        ObjectOutputStream.PutField putFields = objectOutputStream.putFields();
        putFields.put("hgap", this.hgap);
        putFields.put("vgap", this.vgap);
        putFields.put("vector", this.vector);
        putFields.put("currentCard", this.currentCard);
        putFields.put("tab", hashtable);
        objectOutputStream.writeFields();
    }
    
    static {
        serialPersistentFields = new ObjectStreamField[] { new ObjectStreamField("tab", Hashtable.class), new ObjectStreamField("hgap", Integer.TYPE), new ObjectStreamField("vgap", Integer.TYPE), new ObjectStreamField("vector", Vector.class), new ObjectStreamField("currentCard", Integer.TYPE) };
    }
    
    class Card implements Serializable
    {
        static final long serialVersionUID = 6640330810709497518L;
        public String name;
        public Component comp;
        
        public Card(String name, Component comp) {
            this.name = name;
            this.comp = comp;
        }
    }
}
